#!/usr/bin/env python
# coding=utf-8
"""
"""


import numpy as np
from netCDF4 import Dataset, num2date
import syntool_converter.utils.syntoolformat as stfmt
import os


def test_roughness():
    """
    """
    import matplotlib.pyplot as plt
    import pdb
    infile = '/mnt/data/cfosat/data/simulation/iwwoc/Cycle_13_jours_Nominal_MacroCycle/l2s/CFO_T048O1_SWI_L2S____F_20160823T000000_20160823T014500/CFO_T048O1_SWI_L2S10__F_20160823T000000_20160823T014500.nc'
    dset = Dataset(infile)
    sigma0_fluct = dset.variables['sigma0_fluctuation'][:]
    nv = dset.variables['sigma0_nvar'][:]
    dset.close()

    fac = 4
    if fac != 1:
        shp = sigma0_fluct.shape
        siz = shp[1] - np.mod(shp[1], fac)
        rshp = (shp[0], siz / 4, 4)
        sigma0_fluct = sigma0_fluct[:, :siz].reshape(rshp).mean(axis=-1)

    nv_amax = nv.argmax()

    #for i in range(-13, 14):
    # for i in [-13, 0, 13]:
    #     plt.plot(sigma0_fluct[nv_amax + i, :], '+-')
    # plt.show()
    pdb.set_trace()

    #sli = slice(nv_amax - 52, nv_amax + 53)
    sli = slice(0, sigma0_fluct.shape[0])
    rou1 = sigma0_fluct[sli, :]
    rou2 = np.sqrt(rou1 + 1.)
    rou3 = np.zeros(rou1.shape, dtype='uint8')
    rou4 = np.zeros(rou1.shape, dtype='uint8')
    for i in range(rou1.shape[0]):
        var = rou1[i, :]
        vmin, vmax = np.percentile(var, [0.1, 99.9])
        scale, offset = (vmax - vmin) / 254., vmin
        rou3[i, :] = np.clip(np.round((var - offset) / scale), 0, 254).astype('uint8')
        var = rou2[i, :]
        vmin, vmax = np.percentile(var, [0.1, 99.9])
        scale, offset = (vmax - vmin) / 254., vmin
        rou4[i, :] = np.clip(np.round((var - offset) / scale), 0, 254).astype('uint8')
    plt.figure()
    plt.imshow(rou1, interpolation='nearest',
               aspect='auto', cmap=plt.get_cmap('Greys_r'))
    plt.colorbar()
    plt.figure()
    plt.imshow(rou2, interpolation='nearest',
               aspect='auto', cmap=plt.get_cmap('Greys_r'))
    plt.colorbar()
    plt.figure()
    plt.imshow(rou3, interpolation='nearest',
               aspect='auto', cmap=plt.get_cmap('Greys_r'))
    plt.colorbar()
    plt.figure()
    plt.imshow(rou4, interpolation='nearest',
               aspect='auto', cmap=plt.get_cmap('Greys_r'))
    plt.colorbar()
    plt.show()
    pdb.set_trace()


def test_geo():
    """
    """
    import matplotlib.pyplot as plt
    from mpl_toolkits.basemap import Basemap
    import pdb
    infile = '/mnt/data/cfosat/data/simulation/iwwoc/Cycle_13_jours_Nominal_MacroCycle/l2s/CFO_T048O1_SWI_L2S____F_20160823T000000_20160823T014500/CFO_T048O1_SWI_L2S10__F_20160823T000000_20160823T014500.nc'
    dset = Dataset(infile)
    lat = np.stack((dset.variables['lat_near'][:],
                    dset.variables['lat'][:],
                    dset.variables['lat_far'][:]), axis=1)
    lon = np.stack((dset.variables['lon_near'][:],
                    dset.variables['lon'][:],
                    dset.variables['lon_far'][:]), axis=1)
    dset.close()
    bmap = Basemap(resolution='c')
    bmap.fillcontinents()
    bmap.plot(lon, lat, '+')
    plt.show()
    pdb.set_trace()


def swiml2s(infile=None, outdir=None,
            process_var='roughness', downfac=4):
    """
    """
    infile = '/mnt/data/cfosat/data/simulation/iwwoc/Cycle_13_jours_Nominal_MacroCycle/l2s/CFO_T048O1_SWI_L2S____F_20160823T000000_20160823T014500/CFO_T048O1_SWI_L2S10__F_20160823T000000_20160823T014500.nc'
    outdir = '/tmp'
    outdir = '/mnt/data/cfosat/test_syntool/conversion'

    # Read
    dset = Dataset(infile)
    beam_angle = dset.product_angle
    if process_var == 'roughness':
        landmask = dset.variables['land_mask'][:]
        ind_cycles = np.where((landmask.all(axis=1) == False) & \
                              (np.ma.getmaskarray(landmask).all(axis=1) == False))[0]
        var = dset.variables['sigma0_fluctuation'][:][ind_cycles, :]
        if downfac != 1:
            shp = var.shape
            siz = shp[1] - np.mod(shp[1], downfac)
            rshp = (shp[0], siz / downfac, downfac)
            var = var[:, :siz].reshape(rshp).mean(axis=-1)
        product = 'SWIM_{:02d}_L2S_roughness'.format(beam_angle)
    elif process_var == 'wavespec':
        var = dset.variables['wave_spectra'][:]
        ind_cycles = np.where(np.ma.getmaskarray(var).all(axis=1) == False)[0]
        var = var[ind_cycles, :]
        product = 'SWIM_{:02d}_L2S_wave_spectra'.format(beam_angle)
    ncycles, nline = var.shape
    nrange = len(dset.dimensions['range'])
    time = dset.variables['time'][:][ind_cycles]
    time_units = dset.variables['time'].units
    lat = np.stack((dset.variables['lat_near'][:][ind_cycles],
                    dset.variables['lat'][:][ind_cycles],
                    dset.variables['lat_far'][:][ind_cycles]), axis=1)
    lon = np.stack((dset.variables['lon_near'][:][ind_cycles],
                    dset.variables['lon'][:][ind_cycles],
                    dset.variables['lon_far'][:][ind_cycles]), axis=1)
    antenna_rpm = dset.antenna_rpm
    nrot = abs(antenna_rpm) * (time - time[0]) / 60. / 1000.
    dset.close()

    # import matplotlib.pyplot as plt
    # plt.plot(lon, lat, '+-')
    # plt.show()
    # import pdb ; pdb.set_trace()

    # Make GCPs
    lon = lon[:, [1]] + np.mod(lon - lon[:, [1]] + 180., 360.) - 180.
    swathwid = 5. / 111.32
    _ind0 = [0, 1, 1]
    _ind1 = [1, 2, 2]
    swathdir = np.arctan2(lat[:, _ind1] - lat[:, _ind0],
                          lon[:, _ind1] - lon[:, _ind0])
    _gcppix = np.array([-25., 0.5, 26.])
    gcplon = lon[:, :, np.newaxis] + (_gcppix[np.newaxis, np.newaxis, :] - 0.5) * \
             swathwid * np.cos(swathdir[:, :, np.newaxis] - np.pi / 2.)
    gcplat = lat[:, :, np.newaxis] + (_gcppix[np.newaxis, np.newaxis, :] - 0.5) * \
             swathwid * np.sin(swathdir[:, :, np.newaxis] - np.pi / 2.)
    gcppix = np.tile(_gcppix[np.newaxis, :], (3, 1))
    _gcplin = np.array([0.5, nrange / 2., nrange - .5]) / nrange * nline
    gcplin = np.tile(_gcplin[:, np.newaxis], (1, 3))
    gcphei = np.zeros(gcplin.shape)
    #import matplotlib.pyplot as plt
    #import pdb ; pdb.set_trace()

    # Write (per cycles)
    datagroup = os.path.splitext(os.path.basename(infile))[0]
    lim0 = np.searchsorted(nrot, nrot - 0.5, side='left')
    lim1 = np.searchsorted(nrot, nrot + 0.5, side='right')
    _vminmax = np.zeros((ncycles, 2))
    for icyc in xrange(ncycles):
    #for icyc in xrange(52):
        print icyc, ind_cycles[icyc]

        # Metadata
        metadata = {}
        _time = num2date(time[icyc], time_units)
        (dtime, time_range) = stfmt.format_time_and_range(_time, _time, units='ms')
        metadata['product_name'] = product
        metadata['datagroup'] = datagroup
        metadata['name'] = '{}-{:05d}'.format(datagroup, ind_cycles[icyc])
        metadata['datetime'] = dtime
        metadata['time_range'] = time_range

        # Geolocation
        geolocation = {}
        geolocation['projection'] = stfmt.format_gdalprojection()
        geolocation['gcps'] = stfmt.format_gdalgcps(gcplon[icyc, :, :], gcplat[icyc, :, :],
                                                    gcphei, gcppix, gcplin)

        # Band
        band = []
        if process_var == 'roughness':
            vmin, vmax = np.percentile(var[lim0[icyc]:lim1[icyc], :], [0.1, 99.9])
        elif process_var == 'wavespec':
            vmin = 0
            #vmax = np.percentile(var[lim0[icyc]:lim1[icyc], :], 99.9)
            vmax = var[lim0[icyc]:lim1[icyc], :].max()
        _vminmax[icyc, :] = [vmin, vmax]
        scale = (vmax - vmin) / 254.
        offset = vmin
        array = np.clip(np.round((var[icyc, :] - offset) / scale), 0, 254).astype('uint8')
        #array[indzero] = 255
        array = array[:, np.newaxis]
        if process_var == 'roughness':
            band.append({'array':array, 'scale':scale, 'offset':offset,
                         'description':'', 'unittype':'',
                         'nodatavalue':255, 'parameter_range':[vmin, vmax]})
        elif process_var == 'wavespec':
            colortable = stfmt.format_colortable('cerbere_wind', vmax=vmax, vmax_pal=vmax,
                                                 vmin=vmin, vmin_pal=vmin)
            band.append({'array':array, 'scale':scale, 'offset':offset,
                         'description':'', 'unittype':'',
                         'nodatavalue':255, 'parameter_range':[vmin, vmax],
                         'colortable':colortable})

        # Write
        tifffile = stfmt.format_tifffilename(outdir, metadata, create_dir=True)
        stfmt.write_geotiff(tifffile, metadata, geolocation, band)
        #stfmt.write_pngkml_proj(tifffile)
    import matplotlib.pyplot as plt
    plt.plot(_vminmax[:, 0], '+-')
    plt.plot(_vminmax[:, 1], '+-')
    plt.show()
    import pdb ; pdb.set_trace()


def swiml2s_v2(infile=None, outdir=None,
               process_var='roughness', downfac=4):
    """
    """
    infile = '/mnt/data/cfosat/data/simulation/iwwoc/Cycle_13_jours_Nominal_MacroCycle/l2s/CFO_T048O1_SWI_L2S____F_20160823T000000_20160823T014500/CFO_T048O1_SWI_L2S10__F_20160823T000000_20160823T014500.nc'
    outdir = '/tmp'
    outdir = '/mnt/data/cfosat/test_syntool/conversion'
    #process_var = 'wavespec'

    # Read
    dset = Dataset(infile)
    beam_angle = dset.product_angle
    if process_var == 'roughness':
        landmask = dset.variables['land_mask'][:]
        ind_cycles = np.where((landmask.all(axis=1) == False) & \
                              (np.ma.getmaskarray(landmask).all(axis=1) == False))[0]
        var = dset.variables['sigma0_fluctuation'][:][ind_cycles, :]
        if downfac != 1:
            shp = var.shape
            siz = shp[1] - np.mod(shp[1], downfac)
            rshp = (shp[0], siz / downfac, downfac)
            var = var[:, :siz].reshape(rshp).mean(axis=-1)
        product = 'SWIM_{:02d}_L2S_roughness_v2'.format(beam_angle)
    elif process_var == 'wavespec':
        var = dset.variables['wave_spectra'][:]
        ind_cycles = np.where(np.ma.getmaskarray(var).all(axis=1) == False)[0]
        var = var[ind_cycles, :]
        product = 'SWIM_{:02d}_L2S_wave_spectra_v2'.format(beam_angle)
    ncycles, npix = var.shape
    nrange = len(dset.dimensions['range'])
    time = dset.variables['time'][:][ind_cycles]
    time_units = dset.variables['time'].units
    lat = np.stack((dset.variables['lat_near'][:][ind_cycles],
                    dset.variables['lat'][:][ind_cycles],
                    dset.variables['lat_far'][:][ind_cycles]), axis=1)
    lon = np.stack((dset.variables['lon_near'][:][ind_cycles],
                    dset.variables['lon'][:][ind_cycles],
                    dset.variables['lon_far'][:][ind_cycles]), axis=1)
    antenna_rpm = dset.antenna_rpm
    nrot = abs(antenna_rpm) * (time - time[0]) / 60. / 1000.
    import pdb ; pdb.set_trace()
    dset.close()

    #
    ind = np.where(ind_cycles == 9884)[0][0]
    sli = slice(ind - 7, ind + 7)
    var = var[sli, :]
    time = time[sli]
    lat = lat[sli, :]
    lon = lon[sli, :]
    datagroup = os.path.splitext(os.path.basename(infile))[0]

    # Metadata
    metadata = {}
    _time = num2date(time, time_units)
    (dtime, time_range) = stfmt.format_time_and_range(_time[0], _time[-1], units='ms')
    metadata['product_name'] = product
    metadata['datagroup'] = datagroup
    metadata['name'] = '{}-{:05d}-{:05d}'.format(datagroup,
                                                 ind_cycles[sli.start],
                                                 ind_cycles[sli.stop - 1])
    metadata['datetime'] = dtime
    metadata['time_range'] = time_range

    # Geolocation
    geolocation = {}
    geolocation['projection'] = stfmt.format_gdalprojection()
    _gcppix = np.array([0.5, nrange / 2., nrange - .5]) / nrange * npix
    gcppix = np.tile(_gcppix[np.newaxis, :], (var.shape[0], 1))
    _gcplin = np.arange(var.shape[0]) + 0.5
    gcplin = np.tile(_gcplin[:, np.newaxis], (1, 3))
    geolocation['gcps'] = stfmt.format_gdalgcps(lon, lat, np.zeros(lon.shape),
                                                gcppix, gcplin)

    # Band
    band = []
    if process_var == 'roughness':
        vmin, vmax = np.percentile(var, [0.1, 99.9])
    elif process_var == 'wavespec':
        vmin = 0
        vmax = var.max()
    scale = (vmax - vmin) / 254.
    offset = vmin
    array = np.clip(np.round((var - offset) / scale), 0, 254).astype('uint8')
    if process_var == 'roughness':
        band.append({'array':array, 'scale':scale, 'offset':offset,
                     'description':'', 'unittype':'',
                     'nodatavalue':255, 'parameter_range':[vmin, vmax]})
    elif process_var == 'wavespec':
        colortable = stfmt.format_colortable('cerbere_wind', vmax=vmax, vmax_pal=vmax,
                                             vmin=vmin, vmin_pal=vmin)
        band.append({'array':array, 'scale':scale, 'offset':offset,
                     'description':'', 'unittype':'',
                     'nodatavalue':255, 'parameter_range':[vmin, vmax],
                     'colortable':colortable})

    # Write
    tifffile = stfmt.format_tifffilename(outdir, metadata, create_dir=True)
    stfmt.write_geotiff(tifffile, metadata, geolocation, band)
    #stfmt.write_pngkml_proj(tifffile)


def test_overlap():
    """
    """
    from swiml2sproc.l1a import open_l1a
    import matplotlib.pyplot as plt
    import pdb
    l1a_path = '/mnt/data/cfosat/data/simulation/cwwic/aviso_plus/simulated_data/'\
               '2017_Jul_release/Cycle_13_jours_Nominal_MacroCycle/L1a/'\
               'CFO_T048O1_SWI_L1A____F_20160823T000000_20160823T014500.nc'
    l1a, macrocycle = open_l1a(l1a_path)
    inc = 10 #6 #8 #10
    time = l1a[inc].get_times()
    phi = np.rad2deg(l1a[inc].get_values('phi'))
    nswath = l1a[inc].get_field('echo_l1a').get_dimsize('n_swath')
    lon = l1a[inc].get_values('lon_l1a')[:, [0, nswath / 2, -1]]
    lat = l1a[inc].get_values('lat_l1a')[:, [0, nswath / 2, -1]]
    meta = l1a[0].get_metadata()
    antenna_rpm = meta['antenna_rotation_speed']

    valid = np.where(~np.ma.getmaskarray(phi))[0]
    _phi = phi[valid].astype('float64')
    _time = time[valid]
    dphi = _phi[1:] - _phi[:-1]
    dtime = _time[1:] - _time[:-1] # time expected in milliseconds
    drot = -antenna_rpm * dtime / 1000. / 60.
    nrot = np.round((drot * 360. - dphi) / 360.)
    dphi += nrot * 360.
    _phi = _phi[0] + np.concatenate(([0], np.cumsum(dphi)))

    if inc == 6:
        phi_sep = 30.
    elif inc == 8:
        phi_sep = 40.
    elif inc == 10:
        phi_sep = 50.
    _phi -= phi_sep
    phi_start = np.ceil(_phi[0] / 360.) * 360.
    phi_stop = np.floor(_phi[-1] / 360.) * 360.
    nphi_lim = np.round(abs(phi_stop - phi_start) / 360.).astype('int') + 1
    phi_lim = np.arange(nphi_lim) * -360. + phi_start
    phi_lim_ind = np.searchsorted(-_phi, -phi_lim, side='left')
    phi_lim_ind[-1] = _phi.size # be sure to include last point

    for irot in range(nphi_lim - 1):
        _ind0 = phi_lim_ind[irot]
        _ind1 = phi_lim_ind[irot + 1]
        if _ind0 == _ind1:
            continue
        val = valid[_ind0: _ind1]
        _lon = lon[val, :]
        _lat = lat[val, :]
        if np.mod(irot, 2) == 0:
            plt.plot(_lon, _lat, 'o-r')
        # else:
        #     plt.plot(_lon, _lat, 'o-b')
    plt.show()
    pdb.set_trace()


def test_overlap_68():
    """
    """
    from swiml2sproc.l1a import open_l1a
    import matplotlib.pyplot as plt
    import pdb
    l1a_path = '/mnt/data/cfosat/data/simulation/cwwic/aviso_plus/simulated_data/'\
               '2017_Jul_release/Cycle_13_jours_Nominal_MacroCycle/L1a/'\
               'CFO_T048O1_SWI_L1A____F_20160823T000000_20160823T014500.nc'
    l1a, macrocycle = open_l1a(l1a_path)
    data = []
    for inc in [6, 8]:

        time = l1a[inc].get_times()
        phi = np.rad2deg(l1a[inc].get_values('phi'))
        nswath = l1a[inc].get_field('echo_l1a').get_dimsize('n_swath')
        lon = l1a[inc].get_values('lon_l1a')[:, [0, nswath / 2, -1]]
        lat = l1a[inc].get_values('lat_l1a')[:, [0, nswath / 2, -1]]
        meta = l1a[0].get_metadata()
        antenna_rpm = meta['antenna_rotation_speed']
        print antenna_rpm

        if inc == 6:
            #ind = np.where(np.abs(phi - 180.) < 5.)[0]
            #ind0 = ind[4] #ind[0]
            ind0 = 513
            sli = slice(ind0 - 15, ind0 + 15)
            t0 = time[ind0]
        elif inc == 8:
            ind = np.where(np.abs(phi - 180.) < 5.)[0]
            _ind0 = ind[np.abs(ind - ind0).argmin()]
            sli = slice(_ind0 - 15, _ind0 + 15)

        _lon = lon[sli, :].T
        _lat = lat[sli, :].T
        _time = np.tile((time[np.newaxis, sli] - t0) / 1000., (3, 1))
        #_time = np.tile(time[np.newaxis, sli] / 1000., (3, 1))
        print sli, _time.min(), _time.max()
        #plt.plot(_lon, _lat, '-k')
        if inc == 6:
            plt.plot(_lon, _lat, ':r')
        elif inc == 8:
            plt.plot(_lon, _lat, ':b')
        #plt.scatter(_lon.flatten(), _lat.flatten(), c=_time.flatten(), cmap=plt.get_cmap('jet'))
        data.append((_lon, _lat, _time))

    vmin = np.array([data[0][2], data[1][2]]).min()
    vmax = np.array([data[0][2], data[1][2]]).max()
    #print vmin, vmax
    for _d in data:
        plt.scatter(_d[0].flatten(), _d[1].flatten(), c=_d[2].flatten(),
                    cmap=plt.get_cmap('jet'), vmin=vmin, vmax=vmax)
    plt.colorbar()
    plt.show()
    import pdb ; pdb.set_trace()


def test_overlap_frontback():
    """
    """
    from swiml2sproc.l1a import open_l1a
    import matplotlib.pyplot as plt
    import pdb
    l1a_path = '/mnt/data/cfosat/data/simulation/cwwic/aviso_plus/simulated_data/'\
               '2017_Jul_release/Cycle_13_jours_Nominal_MacroCycle/L1a/'\
               'CFO_T048O1_SWI_L1A____F_20160823T000000_20160823T014500.nc'
    l1a, macrocycle = open_l1a(l1a_path)

    # for inc in [6, 8, 10]:
    #     nswath = l1a[inc].get_field('echo_l1a').get_dimsize('n_swath')
    #     lon = l1a[inc].get_values('lon_l1a')[:, nswath / 2]
    #     plt.plot(lon, '+-')
    # plt.show()
    # import pdb ; pdb.set_trace()

    cyc_sli = slice(3000, None)
    data = {}
    for inc in [6, 8, 10]:
        time = l1a[inc].get_times()[cyc_sli]
        phi = np.rad2deg(l1a[inc].get_values('phi')[cyc_sli])
        nswath = l1a[inc].get_field('echo_l1a').get_dimsize('n_swath')
        lon = l1a[inc].get_values('lon_l1a')[cyc_sli, [0, nswath / 2, -1]]
        lat = l1a[inc].get_values('lat_l1a')[cyc_sli, [0, nswath / 2, -1]]
        phif = 0.
        _phi = np.mod(phi - phif, 360.) + phif
        indf = np.where((np.abs(_phi - phif) < 10.) & \
                        (np.ma.getmaskarray(_phi) == False))[0]
        phib = 180.
        _phi = np.mod(phi - phib, 360.) + phib
        indb = np.where((np.abs(_phi - phib) < 10.) & \
                        (np.ma.getmaskarray(_phi) == False))[0]
        data[inc] = {'time':time, 'phi':phi, 'lon':lon, 'lat':lat,
                     'indf': indf, 'indb':indb}

    for inc1 in [6, 8, 10]:

        for dir1 in ['front', 'back']:

            if dir1 == 'front':
                ind1 = data[inc1]['indf']
            elif dir1 == 'back':
                ind1 = data[inc1]['indb']

            for inc2 in [6, 8, 10]:

                if inc2 < inc1:
                    continue

                for dir2 in ['front', 'back']:

                    if inc1 == inc2 and (dir1 != 'back' or dir2 != 'front'):
                        continue

                    if dir2 == 'front':
                        ind2 = data[inc2]['indf']
                    elif dir2 == 'back':
                        ind2 = data[inc2]['indb']
                    print ''
                    print inc1, dir1, inc2, dir2

                    for _pass in ['desc', 'asce']:

                        if _pass == 'desc':
                            ind1p = ind1[data[inc1]['lon'][ind1, 1] > 0.]
                            ind2p = ind2[data[inc2]['lon'][ind2, 1] > 0.]
                        elif _pass == 'asce':
                            ind1p = ind1[data[inc1]['lon'][ind1, 1] < 0.]
                            ind2p = ind2[data[inc2]['lon'][ind2, 1] < 0.]

                        for _lat in [0., -30., -60.]:

                            ind1pl = ind1p[np.abs(data[inc1]['lat'][ind1p, 1] - _lat).argmin()]
                            _lat1 = data[inc1]['lat'][ind1pl, 1]
                            ind2pl = ind2p[np.abs(data[inc2]['lat'][ind2p, 1] - _lat1).argmin()]

                            _time1 = data[inc1]['time'][ind1pl]
                            _time2 = data[inc2]['time'][ind2pl]
                            print (_time2 - _time1) / 1000., ind1pl, ind2pl, _pass, _lat

                            lon1 = data[inc1]['lon'][ind1pl - 12:ind1pl + 13, :]
                            lat1 = data[inc1]['lat'][ind1pl - 12:ind1pl + 13, :]
                            time1 = data[inc1]['time'][ind1pl - 12:ind1pl + 13]
                            lon2 = data[inc2]['lon'][ind2pl - 12:ind2pl + 13, :]
                            lat2 = data[inc2]['lat'][ind2pl - 12:ind2pl + 13, :]
                            time2 = data[inc2]['time'][ind2pl - 12:ind2pl + 13]

                            fig = plt.figure(figsize=[9., 6.])
                            plt.plot(lon1.T, lat1.T, '-k')
                            ncyc = lon2.shape[0]
                            dtime = np.zeros(ncyc)
                            for icyc in range(ncyc):
                                indn = ((lon2[[icyc], :] - lon1) ** 2. + \
                                        (lat2[[icyc], :] - lat1) ** 2.).mean(axis=1).argmin()
                                dtime[icyc] = (time2[icyc] - time1[indn]) / 1000.
                            cmap = plt.get_cmap('jet')
                            cnorm = plt.Normalize(vmin=dtime.min(), vmax=dtime.max())
                            sm = plt.cm.ScalarMappable(cmap=cmap, norm=cnorm)
                            for icyc in range(ncyc):
                                plt.plot(lon2[icyc, :], lat2[icyc, :], '-',
                                         color=sm.to_rgba(dtime[icyc]))
                            sm._A = []
                            plt.colorbar(sm)
                            #plt.plot(lon2.T, lat2.T, '-r')
                            plt.axis('equal')
                            figname = '{:02d}_{:02d}_{}_{}'\
                                      '_lat{}_{}.png'.format(inc1, inc2,
                                                             dir1, dir2,
                                                             int(np.round(_lat)),
                                                             _pass)
                            figpath = os.path.join('/mnt/data/cfosat/test_overlap',
                                                   figname)
                            fig.savefig(figpath)
                            plt.close(fig)
                            #import pdb ; pdb.set_trace()

    import pdb ; pdb.set_trace()


def test_overlap_frontback_seq_loop():
    """
    """
    for qlat in [0., -30., -60.]:
        for qpass in ['asce', 'desc']:
            print qlat, qpass
            test_overlap_frontback_seq(qlat=qlat, qpass=qpass)


def test_phi_def():
    """
    """
    from swiml2sproc.l1a import open_l1a
    import matplotlib.pyplot as plt
    import pdb
    l1a_path = '/mnt/data/cfosat/data/simulation/cwwic/aviso_plus/simulated_data/'\
               '2017_Jul_release/Cycle_13_jours_Nominal_MacroCycle/L1a/'\
               'CFO_T048O1_SWI_L1A____F_20160823T000000_20160823T014500.nc'
    l1a, macrocycle = open_l1a(l1a_path)
    inc = 6
    phi1 = np.rad2deg(l1a[inc].get_values('phi')[:])
    phi2 = np.rad2deg(l1a[inc].get_values('phi_azimuth')[:])
    dphi = np.mod(phi2 - phi1 + 180., 360.) - 180.
    plt.plot(dphi, '+-')
    #plt.plot(phi1, '+-r')
    #plt.plot(phi2, '+-g')
    plt.show()


def test_overlap_frontback_seq(qlat=0., qpass='asce'):
    """
    """
    from swiml2sproc.l1a import open_l1a
    import matplotlib.pyplot as plt
    import pdb
    l1a_path = '/mnt/data/cfosat/data/simulation/cwwic/aviso_plus/simulated_data/'\
               '2017_Jul_release/Cycle_13_jours_Nominal_MacroCycle/L1a/'\
               'CFO_T048O1_SWI_L1A____F_20160823T000000_20160823T014500.nc'
    l1a, macrocycle = open_l1a(l1a_path)

    # for inc in [6, 8, 10]:
    #     nswath = l1a[inc].get_field('echo_l1a').get_dimsize('n_swath')
    #     lon = l1a[inc].get_values('lon_l1a')[:, nswath / 2]
    #     plt.plot(lon, '+-')
    # plt.show()
    # import pdb ; pdb.set_trace()

    cyc_sli = slice(3000, None)
    data = {}
    _qlat = qlat
    for inc in [6, 8, 10]:
        time = l1a[inc].get_times()[cyc_sli]
        phi = np.rad2deg(l1a[inc].get_values('phi')[cyc_sli])
        nswath = l1a[inc].get_field('echo_l1a').get_dimsize('n_swath')
        lon = l1a[inc].get_values('lon_l1a')[cyc_sli, [0, nswath / 2, -1]]
        lat = l1a[inc].get_values('lat_l1a')[cyc_sli, [0, nswath / 2, -1]]
        data[inc] = {'time':time, 'phi':phi, 'lon':lon, 'lat':lat}
        for phid in [180., 0.]:
            if phid == 0.:
                key = 'F'
                _phi = np.mod(phi + 180., 360.) - 180.
            elif phid == 180.:
                key = 'B'
                _phi = np.mod(phi, 360.)
            ind = np.where((np.abs(_phi - phid) < 10.) & \
                           (np.ma.getmaskarray(_phi) == False))[0]
            if qpass == 'desc':
                ind = ind[lon[ind, 1] > 0.]
            elif qpass == 'asce':
                ind = ind[lon[ind, 1] < 0.]
            ind = ind[np.abs(lat[ind, 1] - _qlat).argmin()]
            ind = np.array([ind - 1, ind, ind + 1])
            ind = ind[np.abs(_phi[ind] - phid).argmin()]
            #print _qlat, _phi[ind]
            sli = slice(ind - 5, ind + 6)
            data[inc][key] = {'ind':ind, 'sli':sli, 'time': time[ind],
                              'lon':lon[sli, :], 'lat':lat[sli, :]}
            if inc == 6 and key == 'B':
                _qlat = lat[ind, 1]

    _ind = data[6]['B']['ind']
    print data[6]['phi'][_ind], data[8]['phi'][_ind], data[8]['phi'][_ind] - data[6]['phi'][_ind]
    #import pdb ; pdb.set_trace()
                
    _time = np.array([data[inc][_dir]['time'] for inc in [6, 8, 10] for _dir in ['F', 'B']])
    _lon = np.array([data[inc][_dir]['lon'] for inc in [6, 8, 10] for _dir in ['F', 'B']])
    _lat = np.array([data[inc][_dir]['lat'] for inc in [6, 8, 10] for _dir in ['F', 'B']])
    cmap = plt.get_cmap('jet')
    t0 = _time.min()
    cnorm = plt.Normalize(vmin=0., vmax=(_time.max() - t0) / 1000.)
    sm = plt.cm.ScalarMappable(cmap=cmap, norm=cnorm)
    dll = 0.05 #0.10
    bbox = [_lon.min() - dll, _lon.max() + dll, _lat.min() - dll, _lat.max() + dll]
    order = ['10F', '6F', '8F', '6B', '8B', '10B']
    drots = [1./4, 1./4, 5./4, 1./4, 1./2]
    fig = plt.figure(figsize=[15., 10.])
    for i in range(6):
        inc = int(order[i][:-1])
        _dir = order[i][-1]
        col = sm.to_rgba((data[inc][_dir]['time'] - t0) / 1000.)
        lon = data[inc][_dir]['lon'].T
        lat = data[inc][_dir]['lat'].T
        plt.plot(lon, lat, '-', color=col)
        plt.plot(lon[:, lon.shape[1] / 2], lat[:, lon.shape[1] / 2],
                 '-', color=col, linewidth=3)
    nadtime = l1a[0].get_times()[:]
    nadnswath = l1a[0].get_field('echo_l1a').get_dimsize('n_swath')
    nadlon = l1a[0].get_values('lon_l1a')[:, nadnswath / 2]
    nadlat = l1a[0].get_values('lat_l1a')[:, nadnswath / 2]
    nadind = np.where((nadtime >= _time.min()) & (nadtime <= _time.max()))
    nadlon = nadlon[nadind]
    nadlat = nadlat[nadind]
    plt.plot(nadlon, nadlat, ':k')
    sm._A = []
    plt.colorbar(sm, label='time from 10F [s]', shrink=0.66)
    plt.xlim(bbox[0:2])
    plt.ylim(bbox[2:4])
    plt.axis('equal')
    plt.xlabel('lon [deg]')
    plt.ylabel('lat [deg]')
    ax = plt.gca()
    trans = ax.transAxes
    #trans = ax.transData
    from matplotlib import transforms
    for i in range(6):
        inc = int(order[i][:-1])
        _dir = order[i][-1]
        col = sm.to_rgba((data[inc][_dir]['time'] - t0) / 1000.)
        _text = '{:02d}{}'.format(inc, _dir)
        text = plt.text(0.0, 1.01, _text, color=col, transform=trans, fontsize=12)
        text.draw(fig.canvas.get_renderer())
        ex = text.get_window_extent()
        trans = transforms.offset_copy(text._transform, x=ex.width, units='dots')
        if i != 5:
            inc2, dir2 = int(order[i + 1][:-1]), order[i + 1][-1]
            dt = (data[inc2][dir2]['time'] - data[inc][_dir]['time']) / 1000.
            drot = drots[i]
            _text = ' +{:0.2f}s({:0.2f}R) '.format(dt, drot)
            text = plt.text(0.0, 1.01, _text, color='k', transform=trans, fontsize=12)
            text.draw(fig.canvas.get_renderer())
            ex = text.get_window_extent()
            trans = transforms.offset_copy(text._transform, x=ex.width, units='dots')
    plt.xlim(bbox[0:2])
    plt.ylim(bbox[2:4])
    figdir = '/mnt/data/cfosat/test_overlap_seq'
    figname = 'lat{}_{}.png'.format(np.round(qlat).astype('int'), qpass)
    fig.savefig(os.path.join(figdir, figname), bbox_inches='tight')
    plt.close(fig)
    #plt.show()
    #import pdb ; pdb.set_trace()


def test_phi_shift():
    """
    """
    from swiml2sproc.l1a import open_l1a
    import matplotlib.pyplot as plt
    import pdb
    l1a_path = '/mnt/data/cfosat/data/simulation/cwwic/aviso_plus/simulated_data/'\
               '2017_Jul_release/Cycle_13_jours_Nominal_MacroCycle/L1a/'\
               'CFO_T048O1_SWI_L1A____F_20160823T000000_20160823T014500.nc'
    l1a, macrocycle = open_l1a(l1a_path)

    for i, inc in enumerate([8, 10]):
        time = l1a[inc].get_times()
        phi = np.rad2deg(l1a[inc].get_values('phi'))
        plt.plot(time / 1000., phi, '+-', color=['r', 'b'][i])

    plt.show()
    import pdb ; pdb.set_trace()


def fig2rgb(fig):
    """
    """
    fig.canvas.draw()
    #fig.canvas.print_png('/tmp/test_sm_xspec.png')
    wid, hei = fig.canvas.get_width_height()
    rgb = np.fromstring(fig.canvas.tostring_rgb(), dtype=np.uint8)
    rgb.shape = (hei, wid, 3)
    return rgb


def swiml2s_v3(infile=None, outdir=None):
    """
    """
    import matplotlib
    matplotlib.use('agg')
    import matplotlib.pyplot as plt
    import pyproj

    infile = '/mnt/data/cfosat/data/simulation/iwwoc/Cycle_13_jours_Nominal_MacroCycle/l2s/CFO_T048O1_SWI_L2S____F_20160823T000000_20160823T014500/CFO_T048O1_SWI_L2S06__F_20160823T000000_20160823T014500.nc'
    #infile = '/mnt/data/cfosat/data/simulation/iwwoc/Cycle_13_jours_Nominal_MacroCycle/l2s/CFO_T048O1_SWI_L2S____F_20160823T000000_20160823T014500/CFO_T048O1_SWI_L2S08__F_20160823T000000_20160823T014500.nc'
    #infile = '/mnt/data/cfosat/data/simulation/iwwoc/Cycle_13_jours_Nominal_MacroCycle/l2s/CFO_T048O1_SWI_L2S____F_20160823T000000_20160823T014500/CFO_T048O1_SWI_L2S10__F_20160823T000000_20160823T014500.nc'
    #outdir = '/tmp'
    outdir = '/mnt/data/cfosat/test_syntool/conversion'

    # Read
    dset = Dataset(infile)
    time = dset.variables['time'][:]
    time_units = dset.variables['time'].units
    phi_geo = np.rad2deg(dset.variables['phi_geo'][:])
    wave_spec = dset.variables['wave_spectra'][:]
    lat = np.stack((dset.variables['lat_near'][:],
                    dset.variables['lat'][:],
                    dset.variables['lat_far'][:]), axis=1)
    lon = np.stack((dset.variables['lon_near'][:],
                    dset.variables['lon'][:],
                    dset.variables['lon_far'][:]), axis=1)
    k = dset.variables['k'][:]
    label = dset.variables['partition_label'][:]
    antenna_rpm = dset.antenna_rpm
    beam_angle = dset.product_angle
    dset.close()

    # hack for phi (not yet in L2S)
    from swiml2sproc.l1a import open_l1a
    l1a_path = '/mnt/data/cfosat/data/simulation/cwwic/aviso_plus/simulated_data/'\
               '2017_Jul_release/Cycle_13_jours_Nominal_MacroCycle/L1a/'\
               'CFO_T048O1_SWI_L1A____F_20160823T000000_20160823T014500.nc'
    l1a, macrocycle = open_l1a(l1a_path)
    phi = np.rad2deg(l1a[beam_angle].get_values('phi'))

    #valid = np.where(~np.ma.getmaskarray(phi_geo))[0]
    #_phi = phi_geo[valid].astype('float64')
    valid = np.where(~np.ma.getmaskarray(phi))[0]
    _phi = phi[valid].astype('float64')
    _time = time[valid]
    dphi = _phi[1:] - _phi[:-1]
    dtime = _time[1:] - _time[:-1] # time expected in milliseconds
    drot = -antenna_rpm * dtime / 1000. / 60.
    nrot = np.round((drot * 360. - dphi) / 360.)
    dphi += nrot * 360.
    _phi = _phi[0] + np.concatenate(([0], np.cumsum(dphi)))

    if beam_angle == 6:
        phi_sep = 30.
    elif beam_angle == 8:
        phi_sep = 40.
    elif beam_angle == 10:
        phi_sep = 50.
    _phi -= phi_sep
    phi_start = np.ceil(_phi[0] / 360.) * 360.
    phi_stop = np.floor(_phi[-1] / 360.) * 360.
    nphi_lim = np.round(abs(phi_stop - phi_start) / 360.).astype('int') + 1
    phi_lim = np.arange(nphi_lim) * -360. + phi_start
    phi_lim_ind = np.searchsorted(-_phi, -phi_lim, side='left')
    phi_lim_ind[-1] = _phi.size # be sure to include last point

    # diff = np.mod(_phi - phi_geo[valid] + 180., 360.) - 180.
    # plt.plot(diff, '+-')
    # plt.show()
    # import pdb ; pdb.set_trace()

    geod = pyproj.Geod(ellps='WGS84')
    cmap, _ = stfmt.load_colormap('cerbere_wind')
    product = 'SWIM_{:02d}_L2S_wave_spectrum'.format(beam_angle)
    bname = os.path.splitext(os.path.basename(infile))[0]
    klim = 2. * np.pi / np.array([400., 200., 100.])
    kpos = np.searchsorted(k, klim, side='left')
    for irot in range(nphi_lim - 1):

        _ind0 = phi_lim_ind[irot]
        _ind1 = phi_lim_ind[irot + 1]
        if _ind0 == _ind1:
            continue

        val = valid[_ind0: _ind1]
        spec = wave_spec[val, :]
        nspec = np.sum(np.ma.getmaskarray(spec).any(axis=1) == False)
        if nspec < 49:
            continue
        lab = label[val, :]

        _lon0 = lon[val, 0]
        _lon1 = lon[val, 2]
        _lat0 = lat[val, 0]
        _lat1 = lat[val, 2]
        # if np.abs(_lat0).max() < 60.:
        #     continue
        lon0 = np.concatenate(([_lon0[0] - (_lon0[1] - _lon0[0]) / 2.],
                               (_lon0[:-1] + _lon0[1:]) / 2.,
                               [_lon0[-1] + (_lon0[-1] - _lon0[-2]) / 2.]))
        lon1 = np.concatenate(([_lon1[0] - (_lon1[1] - _lon1[0]) / 2.],
                               (_lon1[:-1] + _lon1[1:]) / 2.,
                               [_lon1[-1] + (_lon1[-1] - _lon1[-2]) / 2.]))
        lat0 = np.concatenate(([_lat0[0] - (_lat0[1] - _lat0[0]) / 2.],
                               (_lat0[:-1] + _lat0[1:]) / 2.,
                               [_lat0[-1] + (_lat0[-1] - _lat0[-2]) / 2.]))
        lat1 = np.concatenate(([_lat1[0] - (_lat1[1] - _lat1[0]) / 2.],
                               (_lat1[:-1] + _lat1[1:]) / 2.,
                               [_lat1[-1] + (_lat1[-1] - _lat1[-2]) / 2.]))

        _lon = np.zeros((spec.shape[0] + 1, spec.shape[1] + 1))
        _lat = np.zeros((spec.shape[0] + 1, spec.shape[1] + 1))
        for icyc in range(spec.shape[0] + 1):
            _ll = np.array(geod.npts(lon0[icyc], lat0[icyc],
                                     lon1[icyc], lat1[icyc],
                                     spec.shape[1] + 1, radians=False))
            _lon[icyc, :] = _ll[:, 0]
            _lat[icyc, :] = _ll[:, 1]

        # plt.plot(lon0, lat0, '+-')
        # plt.plot(lon1, lat1, '+-')
        # plt.plot(_lon, _lat, 'x-')
        # plt.show()
        # import pdb ; pdb.set_trace()

        # from mpl_toolkits.basemap import Basemap
        # bmap = Basemap(llcrnrlon=_lon.min(), llcrnrlat=_lat.min(),
        #                urcrnrlon=_lon.max(), urcrnrlat=_lat.max(),
        #                resolution='l')
        # bmap.fillcontinents()
        # bmap.pcolormesh(_lon, _lat, spec, cmap=cmap)
        # bmap.plot(lon[val, :], lat[val, :], '+-')
        # plt.show()
        # import pdb ; pdb.set_trace()

        #ll_res = 360. / (256. * 2. ** 9)
        ll_res = 360. / (256. * 2. ** 10)
        ll_bbox = [_lon.min(), _lat.min(), _lon.max(), _lat.max()]
        figsize = np.ceil(np.array([ll_bbox[2] - ll_bbox[0],
                                    ll_bbox[3] - ll_bbox[1]]) / ll_res).astype('int')
        ll_bbox2 = [ll_bbox[0], ll_bbox[1], ll_bbox[0] + figsize[0] * ll_res,
                    ll_bbox[1] + figsize[1] * ll_res]
        fig = plt.figure(figsize=figsize / 100., dpi=100, facecolor='k')
        ax = fig.add_axes([0., 0., 1., 1.], facecolor='k')
        ax.set_xlim([ll_bbox2[0], ll_bbox2[2]])
        ax.set_ylim([ll_bbox2[1], ll_bbox2[3]])
        ax.pcolormesh(_lon, _lat, spec, cmap=cmap, vmin=0.)
        xpsep, ypsep = np.where(lab[:, :-1] != lab[:, 1:])
        if xpsep.size != 0:
            x = np.stack((xpsep, xpsep + 1), axis=0)
            y = np.stack((ypsep + 1, ypsep + 1), axis=0)
            ax.plot(_lon[x, y], _lat[x, y], '-', color='0.1')
        xpsep, ypsep = np.where(lab[:-1, :] != lab[1:, :])
        if xpsep.size != 0:
            x = np.stack((xpsep + 1, xpsep + 1), axis=0)
            y = np.stack((ypsep, ypsep + 1), axis=0)
            ax.plot(_lon[x, y], _lat[x, y], '-', color='0.1')
        xpsep = np.where(lab[:, 0] != 0)[0]
        if xpsep.size != 0:
            x = np.stack((xpsep, xpsep +1), axis=0)
            ax.plot(_lon[x, 0], _lat[x, 0], '-', color='0.1')
            #print '0', irot
        xpsep = np.where(lab[:, -1] != 0)[0]
        if xpsep.size != 0:
            x = np.stack((xpsep, xpsep +1), axis=0)
            ax.plot(_lon[x, -1], _lat[x, -1], '-', color='0.1')
            #print '-1', irot
        # TODO for partition: start/end of rotation
        for _kpos in kpos:
            ax.plot(_lon[:, _kpos], _lat[:, _kpos], ':', color='0.1')

        #fig.savefig('/tmp/test.png')
        rgb = fig2rgb(fig)
        plt.close(fig)

        # Metadata
        metadata = {}
        _time0 = num2date(time[val[0]], time_units)
        _time1 = num2date(time[val[1]], time_units)
        (dtime, time_range) = stfmt.format_time_and_range(_time0, _time1, units='ms')
        metadata['product_name'] = product
        metadata['datagroup'] = '{}-{}'.format(bname, np.mod(irot, 2))
        metadata['name'] = '{}-{:03d}'.format(bname, irot)
        metadata['datetime'] = dtime
        metadata['time_range'] = time_range

        # Geolocation
        geolocation = {}
        geolocation['projection'] = stfmt.format_gdalprojection()
        geolocation['geotransform'] = [ll_bbox2[0], ll_res, 0.,
                                       ll_bbox2[3], 0., -ll_res]

        # Band
        band = []
        band.append({'array': rgb[:, :, 0], 'nodatavalue':0})
        band.append({'array': rgb[:, :, 1], 'nodatavalue':0})
        band.append({'array': rgb[:, :, 2], 'nodatavalue':0})

        # Write
        tifffile = stfmt.format_tifffilename(outdir, metadata, create_dir=True)
        stfmt.write_geotiff(tifffile, metadata, geolocation, band)
        #import pdb ; pdb.set_trace()
        #break


