#!/usr/bin/env python
# coding=utf-8
"""
"""

import matplotlib
matplotlib.use('agg')

import matplotlib.pyplot as plt
import numpy as np
from netCDF4 import Dataset, num2date
import os
import pyproj

try:
    from osgeo import osr
except ImportError:
    import osr

import syntool_converter.utils.syntoolformat as stfmt


def swim_l2s_spec(infile=None, outdir=None, kind='fluctuation', nrot=3, use_klog=True):
    """
    """
    if infile is None:
        #infile = '/run/media/gilles/WD_GG/cfosat/swi_l2s___/v0.2.0/2019/021/CFO_OPER_SWI_L2S____F_20190121T180511_20190121T194915_0.2/CFO_OPER_SWI_L2S02__F_20190121T180511_20190121T194915_0.2.nc'
        #infile = '/run/media/gilles/WD_GG/cfosat/swi_l2s___/v0.2.0/2019/021/CFO_OPER_SWI_L2S____F_20190121T180511_20190121T194915_0.2/CFO_OPER_SWI_L2S04__F_20190121T180511_20190121T194915_0.2.nc'
        #infile = '/run/media/gilles/WD_GG/cfosat/swi_l2s___/v0.2.0/2019/021/CFO_OPER_SWI_L2S____F_20190121T180511_20190121T194915_0.2/CFO_OPER_SWI_L2S06__F_20190121T180511_20190121T194915_0.2.nc'
        #infile = '/run/media/gilles/WD_GG/cfosat/swi_l2s___/v0.2.0/2019/021/CFO_OPER_SWI_L2S____F_20190121T180511_20190121T194915_0.2/CFO_OPER_SWI_L2S08__F_20190121T180511_20190121T194915_0.2.nc'
        infile = '/run/media/gilles/WD_GG/cfosat/swi_l2s___/v0.2.0/2019/021/CFO_OPER_SWI_L2S____F_20190121T180511_20190121T194915_0.2/CFO_OPER_SWI_L2S10__F_20190121T180511_20190121T194915_0.2.nc'
    if outdir is None:
        #outdir = '/tmp'
        #outdir = '/run/media/gilles/WD_GG/cfosat/syntool/tests/test1'
        outdir = '/run/media/gilles/WD_GG/cfosat/syntool/tests/test4'

    # Read
    dset = Dataset(infile)
    beam_angle = dset.l2s_angle
    if kind == 'fluctuation':
        if beam_angle == 2:
            spec = dset.variables['fluctuation_spectra'][:, :, 10:].mean(axis=-1)
        else:
            spec = dset.variables['fluctuation_spectra'][:].mean(axis=-1)
        k = dset.variables['klin'][:]
    elif kind == 'modulation':
        raise Exception('not yet tested')
        if beam_angle == 2:
            spec = dset.variables['modulation_spectra'][:, :, 10:].mean(axis=-1)
        else:
            spec = dset.variables['modulation_spectra'][:].mean(axis=-1)
        k = dset.variables['klin'][:]
    elif kind == 'wave':
        raise Exception('not yet tested')
        spec = dset.variables['wave_spectra'][:]
        k = dset.variables['k'][:]
        label = dset.variables['partition_label'][:]
    else:
        raise Exception('which kind of spectra ?')
    time = dset.variables['time'][:]
    time_units = dset.variables['time'].units # should be seconds
    lat = np.ma.stack((dset.variables['near_lat'][:],
                       dset.variables['lat'][:],
                       dset.variables['far_lat'][:]), axis=1)
    lon = np.ma.stack((dset.variables['near_lon'][:],
                       dset.variables['lon'][:],
                       dset.variables['far_lon'][:]), axis=1)
    if beam_angle == 2:
        lat[:, 0] = dset.variables['seg_lat'][:, 10]
        lon[:, 0] = dset.variables['seg_lon'][:, 10]
    phi = dset.variables['phi'][:]
    antenna_rpm = np.abs(dset.antenna_rpm)
    seg_flag = dset.variables['seg_flag'][:]
    dset.close()

    if not time_units.lower().startswith('seconds'):
        raise Exception('Expects time in seconds')

    # Graphic parameters
    merc = True
    #merc = False
    if merc:
        x = np.ma.masked_all(lon.shape, dtype=lon.dtype)
        y = np.ma.masked_all(lon.shape, dtype=lon.dtype)
        indll = np.where((np.ma.getmaskarray(lon) == False) & (np.ma.getmaskarray(lat) == False))
        pmerc = pyproj.Proj(init='epsg:3857')
        _x, _y = pmerc(lon[indll], lat[indll])
        x[indll] = _x
        y[indll] = _y
        x_ext = 20037508.34 * 2.
        y_ext = 20037508.34 * 2.
        srs = osr.SpatialReference()
        srs.ImportFromEPSG(3857)
    else:
        x = lon.copy()
        y = lat.copy()
        x_ext = 360.
        y_ext = 180.
        srs = osr.SpatialReference()
        srs.SetWellKnownGeogCS('WGS84')
    zoom = 10
    res = x_ext / (256. * 2. ** zoom)
    if beam_angle == 2:
        _wid = 10
    elif beam_angle == 4:
        _wid = 13
    elif beam_angle == 6:
        _wid = 16
    elif beam_angle == 8:
        #_wid = 24
        _wid = 20
    elif beam_angle == 10:
        #_wid = 32
        _wid = 24
    wid = _wid * res
    cmap, _ = stfmt.load_colormap('cerbere_wind')
    klimlinwid = 2.

    # k selection
    if use_klog == False:
        indk = np.where(k <= 2. * np.pi / 100.)[0]
        slik = slice(indk[0], indk[-1] + 1)
        nk = indk.size
        dk = k[1] - k[0]
        k0 = k[indk[0]] - dk / 2.
        k1 = k[indk[-1]] + dk / 2.
        wllim = np.array([400., 200.])
        poslim = (2. * np.pi / wllim - k0) / (k1 - k0)
    else:
        _spec = spec.copy()
        _k = k.copy()
        nk = 26
        slik = slice(0, nk)
        k0 = 2. * np.pi / 1000.
        k1 = 2. * np.pi / 50.
        alpha = (k1 / k0) ** (1. / nk)
        k = k0 * alpha ** (np.arange(nk) + 0.5)
        ## Interpolate spectra
        # klims = k0 * alpha ** (np.arange(nk + 1))
        # spec = np.ma.masked_all((spec.shape[0], k.size), dtype=spec.dtype)
        # _dk = _k[1] - _k[0]
        # _klims = np.concatenate((_k - _dk / 2., [_k[-1] + _dk / 2.]))
        # for i in range(nk):
        #     _indk = np.where((_klims[1:] > klims[i]) & \
        #                      (_klims[:-1] < klims[i + 1]))[0]
        #     if _indk.size == 0:
        #         continue
        #     _dki = np.minimum(_klims[_indk + 1], klims[i + 1]) - \
        #         np.maximum(_klims[_indk], klims[i])
        #     _speci = _spec[:, _indk] * _dki[np.newaxis, :]
        #     spec[:, i] = _speci.sum(axis=1) / (klims[i + 1] - klims[i])
        indint = np.searchsorted(_k, k, side='right') - 1
        intnorm = (k - _k[indint]) / (_k[indint + 1] - _k[indint])
        spec = _spec[:, indint] + (_spec[:, indint + 1] - _spec[:, indint]) * intnorm[np.newaxis, :]
        # import matplotlib.pyplot as plt
        # plt.plot(_k, _spec[831, :], '+-')
        # plt.plot(k, spec[831, :], '+-')
        # plt.show()
        # import pdb ; pdb.set_trace()

        wllim = np.array([800., 400., 200., 100.])
        klim = 2. * np.pi / wllim
        ik = (np.arange(nk) + 0.5) / nk
        indint = np.searchsorted(k, klim, side='right') - 1
        intnorm = (klim - k[indint]) / (k[indint + 1] - k[indint])
        poslim = ik[indint] + (ik[indint + 1] - ik[indint]) * intnorm

    # Contrast
    ## Disable high latitudes
    indlat = np.where(np.abs(lat[:, 1]) >= 80.)[0]
    spec[indlat, :] = np.ma.masked
    ## vmin/vmax
    ### auto
    # _indk = np.where((k >= 2. * np.pi / 600.) & (k <= 2. * np.pi / 100.))[0]
    # _slik = slice(_indk[0], _indk[-1] + 1)
    # _okphi = ((phi >= 22.5) & (phi <= 157.5)) | ((phi >= 202.5) & (phi <= 337.5))
    # _okflag = seg_flag.max(axis=1) == 1
    # #_indcyc = np.where(_okphi)[0]
    # _indcyc = np.where(_okphi & _okflag)[0]
    # vmin, vmax = np.percentile(spec[_indcyc, _slik].compressed(), (1., 99.95))
    ## \HISTO
    # his1, edg1 = np.histogram(spec[_indcyc, _slik].compressed(), bins=500)
    # his2, edg2 = np.histogram(np.ma.log(spec[_indcyc, _slik]).compressed(), bins=500)
    # fig = plt.figure()
    # plt.plot((edg1[1:] + edg1[:-1]) / 2., his1, '+-')
    # plt.plot([vmin] * 2, plt.ylim(), ':k')
    # plt.plot([vmax] * 2, plt.ylim(), ':k')
    # fig = plt.figure()
    # plt.plot((edg2[1:] + edg2[:-1]) / 2., his2, '+-')
    # plt.plot([np.log(vmin)] * 2, plt.ylim(), ':k')
    # plt.plot([np.log(vmax)] * 2, plt.ylim(), ':k')
    # plt.show()
    # print beam_angle, vmin, vmax
    # import pdb ; pdb.set_trace()
    ## \HISTO
    ### fixed
    ### based on auto with CFO_OPER_SWI_L2S____F_20190121T180511_20190121T194915_0.2
    if beam_angle == 2:
        vmin, vmax = 0.017007010323660716, 0.6303030221121608
    elif beam_angle == 4:
        vmin, vmax = 0.012579781668526786, 0.5347529820033482
    elif beam_angle == 6:
        vmin, vmax = 0.005074055989583334, 0.23380649617513546
    elif beam_angle == 8:
        vmin, vmax = 0.00335693359375, 0.17073221015929851
    elif beam_angle == 10:
        vmin, vmax = 0.002567545572916667, 0.1576303792317423
    spec = np.ma.log(np.ma.maximum(spec, vmin))
    vmin, vmax = np.log(vmin), np.log(vmax)

    # Orbit cutting
    indt = np.where(~np.ma.getmaskarray(time))[0]
    tmin = time[indt].min()
    tmax = time[indt].max()
    cut_dtime = 60. / antenna_rpm * nrot
    # ncut = np.ceil((tmax - tmin) / cut_dtime).astype('int')
    # cut_time = tmin + np.linspace(0., ncut * cut_dtime, num=ncut + 1)
    # cut_ind = np.searchsorted(time[indt], cut_time, side='left')
    phiref = 90.
    ##phiref = 0.
    phifilled = np.ma.filled(phi[indt], phiref + 180.) # in case phi are masked
    phimod = np.mod(phifilled - phiref + 180., 360.) + phiref - 180.
    indref = np.abs(phimod - phiref).argmin()
    tref = time[indt[indref]]
    nbef = np.ceil((tref - tmin) / cut_dtime).astype('int')
    naft = np.ceil((tmax - tref) / cut_dtime).astype('int')
    cut_time = np.arange(-nbef, naft + 1) * cut_dtime + tref
    ncut = cut_time.size - 1
    cut_ind = np.searchsorted(time[indt], cut_time, side='left')
    cut_ind[0] = 0
    cut_ind[-1] = indt.size
    # for icut in range(ncut):
    #     print phi[indt[cut_ind[icut]]]
    # import pdb ; pdb.set_trace()

    # Loop on cuts
    product = 'SWIM_{:02d}_L2S_{}_spectrum'.format(beam_angle, kind)
    bname = os.path.splitext(os.path.basename(infile))[0]
    cntcut = 0
    for icut in range(ncut):

        _indt = indt[cut_ind[icut]: cut_ind[icut + 1]]
        if _indt.size == 0:
            continue

        _spec = spec[_indt, :]
        _inds = np.where(np.ma.getmaskarray(_spec).max(axis=1) == False)[0]
        if _inds.size == 0:
            continue

        _y = y[_indt, :]
        _x = x[_indt, :].copy()
        if (_x.max() - _x.min()) >= x_ext / 2.:
            _x = np.ma.mod(_x, x_ext)

        unit = np.linspace(0., 1., num=nk + 1, endpoint=True)[np.newaxis, :]
        dx = (_x[_inds, 2] - _x[_inds, 0])[:, np.newaxis]
        xctr = _x[_inds, 0][:, np.newaxis] + dx * unit
        xctrlim = _x[_inds, 0][:, np.newaxis] + dx * poslim[np.newaxis, :]
        dy = (_y[_inds, 2] - _y[_inds, 0])[:, np.newaxis]
        yctr = _y[_inds, 0][:, np.newaxis] + dy * unit
        yctrlim = _y[_inds, 0][:, np.newaxis] + dy * poslim[np.newaxis, :]
        dxy = np.sqrt(dx ** 2. + dy ** 2.)
        dx = dx / dxy
        dy = dy / dxy
        xedg = np.stack((xctr - dy * wid / 2., xctr + dy * wid / 2.), axis=2)
        yedg = np.stack((yctr + dx * wid / 2., yctr - dx * wid / 2.), axis=2)
        xedglim = np.stack((xctrlim - dy * wid / 2., xctrlim + dy * wid / 2.), axis=2)
        yedglim = np.stack((yctrlim + dx * wid / 2., yctrlim - dx * wid / 2.), axis=2)

        # Figure
        #bbox = [_x.min(), _y.min(), _x.max(), _y.max()]
        bbox = [xedg.min(), yedg.min(), xedg.max(), yedg.max()]
        if beam_angle <= 4:
            _indt2s = [_indt[_inds[0]], _indt[_inds[-1]]]
            for _indt2, _dind in zip(_indt2s, [-1, +1]):
                _indadd = _indt2 + _dind
                if _indadd < 0 or _indadd >= x.shape[0]:
                    continue
                _tadd = time[_indadd]
                if np.ma.is_masked(_tadd):
                    continue
                if np.abs(_tadd - time[_indt2]) > 60. / antenna_rpm / 4.:
                    continue
                _yadd = y[_indadd, 1]
                if not np.ma.is_masked(_yadd):
                    bbox[1] = min([bbox[1], _yadd])
                    bbox[3] = max([bbox[3], _yadd])
        #plt.plot(np.array(ll_bbox)[[0, 2, 2, 0, 0]], np.array(ll_bbox)[[1, 1, 3, 3, 1]], '-')
        #continue
        figsize = np.ceil(np.array([bbox[2] - bbox[0],
                                    bbox[3] - bbox[1]]) / res).astype('int32')
        bbox2 = [bbox[0], bbox[1], bbox[0] + figsize[0] * res,
                 bbox[1] + figsize[1] * res]
        fig = plt.figure(figsize=figsize / 100. + 0.0001, dpi=100., facecolor='k')
        ax = fig.add_axes([0., 0., 1., 1.], facecolor='k')
        ax.axis('off')
        ax.set_xlim([bbox2[0], bbox2[2]])
        ax.set_ylim([bbox2[1], bbox2[3]])
        for j in range(_inds.size):
            ax.pcolormesh(xedg[j, :, :], yedg[j, :, :],
                          _spec[_inds[j], slik, np.newaxis],
                          cmap=cmap, vmin=vmin, vmax=vmax)
            for ilim in range(poslim.size):
                ax.plot(xedglim[j, ilim, :], yedglim[j, ilim, :], '-',
                        color='0.1', linewidth=klimlinwid)
        rgb = fig2rgb(fig)
        #print fig, rgb.shape, figsize, figsize / 100., figsize / 100. + 0.0001
        #print fig.bbox.bounds
        plt.close(fig)

        xidl = bbox2[2] > x_ext / 2.
        if not xidl:
            rgbs = [rgb]
            bbox2s = [bbox2]
        else:
            rgbs = []
            bbox2s = []
            fig_nx = rgb.shape[1]
            bbox_x = np.arange(fig_nx + 1) * res + bbox2[0]
            indsplit = np.searchsorted(bbox_x, x_ext / 2., side='right') - 1
            if indsplit > 0: # before x_ext / 2
                rgbs.append(rgb[:, :indsplit, :])
                _bbox2 = list(bbox2)
                _bbox2[2] = bbox_x[indsplit]
                bbox2s.append(_bbox2)
            if indsplit < fig_nx - 1: # after x_ext / 2
                rgbs.append(rgb[:, indsplit + 1:, :])
                _bbox2 = list(bbox2)
                _bbox2[0] = bbox_x[indsplit + 1] - x_ext
                _bbox2[2] = _bbox2[2] - x_ext
                bbox2s.append(_bbox2)
            #print cntcut
            #import pdb ; pdb.set_trace()

        for _bbox2, _rgb in zip(bbox2s, rgbs):

            # Metadata
            metadata = {}
            _time0 = num2date(time[_indt].min(), time_units)
            _time1 = num2date(time[_indt].max(), time_units)
            (dtime, time_range) = stfmt.format_time_and_range(_time0, _time1, units='ms')
            metadata['product_name'] = product
            metadata['datagroup'] = bname
            metadata['name'] = '{}-{:04d}'.format(bname, cntcut)
            metadata['datetime'] = dtime
            metadata['time_range'] = time_range

            # Geolocation
            geolocation = {}
            geolocation['projection'] = srs.ExportToWkt()
            geolocation['geotransform'] = [_bbox2[0], res, 0.,
                                           _bbox2[3], 0., -res]

            # Band
            band = []
            band.append({'array': _rgb[:, :, 0], 'nodatavalue':0})
            band.append({'array': _rgb[:, :, 1], 'nodatavalue':0})
            band.append({'array': _rgb[:, :, 2], 'nodatavalue':0})

            # Write
            tifffile = stfmt.format_tifffilename(outdir, metadata, create_dir=True)
            stfmt.write_geotiff(tifffile, metadata, geolocation, band)

            cntcut += 1

    #plt.show()
    #import pdb ; pdb.set_trace()


def fig2rgb(fig):
    """
    """
    fig.canvas.draw()
    #fig.canvas.print_png('/tmp/fig.png')
    wid, hei = fig.canvas.get_width_height()
    rgb = np.fromstring(fig.canvas.tostring_rgb(), dtype=np.uint8)
    rgb.shape = (hei, wid, 3)
    return rgb

