# -*- encoding: utf-8 -*-

import ssl
import time
import socket
import logging
import httplib

logger = logging.getLogger(__name__)


class HTTPTimeout(Exception):
    """"""
    pass


class HTTPResponseError(Exception):
    """"""
    pass


def download_file(conn, url, method='GET', headers=None, payload=None,
                  tries=1, retry_delay=3):
    """"""
    ok = False
    raw_data = None

    if headers is None:
        headers = {}

    while (ok is False) and 0 < tries:
        tries = tries - 1
        try:
            conn.request(method, url, headers=headers, body=payload)
        except httplib.CannotSendRequest:
            _msg = 'Cannot send request to download file {}'.format(url)
            if 0 >= tries:
                logger.error(_msg)
                raise
            else:
                # Wait and retry
                logger.warning(_msg)
                logger.info('Retrying...')
                time.sleep(retry_delay)
                continue

        try:
            r = conn.getresponse()
        except socket.timeout:
            _msg = 'Timeout while waiting for a response from {}'
            if 0 >= tries:
                logger.error(_msg.format(url))
                raise HTTPTimeout
            else:
                # Wait and retry
                logger.warning(_msg.format(url))
                logger.info('Retrying...')
                time.sleep(retry_delay)
                continue
        except ssl.SSLError:
            _msg = 'SSL Timeout while waiting for a response from {}'
            if 0 >= tries:
                logger.error(_msg.format(url))
                raise HTTPTimeout
            else:
                # Wait and retry
                logger.warning(_msg.format(url))
                logger.info('Retrying...')
                time.sleep(retry_delay)
                continue

        if r.status not in (200, 301, 302,):
            _msg = 'HTTP {} request to {} returned a {} status'
            if 0 >= tries:
                logger.error(_msg.format(method, url, r.status))
                logger.error('Reason: {}'.format(r.reason))
                raise HTTPResponseError
            else:
                try:
                    _ = r.read()
                except:  # noqa
                    pass
                logger.warning(_msg.format(method, url, r.status))
                logger.warning('Reason: {}'.format(r.reason))
                logger.info('Retrying...')
                time.sleep(retry_delay)
                continue

        try:
            raw_data = r.read()
        except httplib.IncompleteRead:
            _msg = ('Could not get the full response from the server '
                    '(incomplete read) for {}')
            if 0 >= tries:
                logger.error(_msg.format(url))
                raise
            else:
                # Wait and retry
                logger.warning(_msg.format(url))
                logger.info('Retrying...')
                time.sleep(retry_delay)
                continue
        except ssl.SSLError:
            _msg = 'SSL Timeout while waiting for a response from {}'
            if 0 >= tries:
                logger.error(_msg.format(url))
                raise HTTPTimeout
            else:
                # Wait and retry
                logger.warning(_msg.format(url))
                logger.info('Retrying...')
                time.sleep(retry_delay)
                continue


        ok = True
        logger.info('download complete')


    return raw_data
