# -*- encoding: utf-8 -*-

"""
Copyright (C) 2014-2021 OceanDataLab

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import numpy
import logging

logger = logging.getLogger(__name__)


def ubytes_0_254(values, vmin, vmax):
    """Pack input values as ubytes between 0 and 254.
    255 is reserved for missing or masked values."""
    offset = vmin
    scale = (vmax - vmin) / 254.0
    if vmin == vmax:
        scale = 1.0

    numpy.clip(values, vmin, vmax, out=values)

    # Required to avoid runtime warnings on masked arrays wherein the
    # division of the _FillValue by the scale cannot be stored by the dtype
    # of the array
    if isinstance(values, numpy.ma.MaskedArray):
        mask = numpy.ma.getmaskarray(values).copy()
        values[numpy.where(mask)] = vmin
        _values = (numpy.ma.getdata(values) - offset) / scale
        values.mask = mask  # Restore mask to avoid side-effects
    else:
        _values = (values - offset) / scale

    result = numpy.round(_values).astype('ubyte')
    return result, offset, scale
